{ Daniel Pereira Guimares
  tecnobyte@ulbrajp.com.br

  Dicas sobre Delphi no IntereSite:
  www.ulbrajp.com.br/~tecnobyte

  Exemplos de uso das funes:

  =====================
  funo tbIsCPF()
  =====================

  - Coloque um Edit e um Button.
  - No OnClick do Button coloque:

    if tbIsCPF(Edit1.Text) then
      ShowMessage('CPF ok!')
    else
      ShowMessage('CPF invlido');

  - Obs: as funes tbIsCGC e tbIsCPForCGC so semelhantes.

  =====================
  funo tbFmtCPF()
  =====================

  - Coloque dois Edit's e um Button.
  - No OnClick do Button coloque:

    Edit2.Text := tbFmtCPF(Edit1.Text);

  - Obs: as funes tbFmtCGC e tbFmtCPForCGC so semelhantes.
}

unit tbCPFCGC;

interface
uses SysUtils;

function tbIsCPF(const CPF: string): boolean;
function tbIsCGC(const CGC: string): boolean;
function tbIsCPForCGC(const S: string): boolean;
function tbFmtCPF(const S: string): string;
function tbFmtCGC(const S: string): string;
function tbFmtCPForCGC(const S: string): string;

implementation

{ Pega somente os dgitos numricos da string }
function tbDigits(const S: string): string;
var
  I: integer;
begin
  Result := '';
  for I := 1 to Length(S) do
    if Pos(S[I], '0123456789') > 0 then
      Result := Result + S[I];
end;

{ Retorna true se a string for um CPF vlido }
function tbIsCPF(const CPF: string): boolean;
var
  I, Soma, Digito: integer;
  CalcCPF, S1, S2: string;
  B: boolean;
  C: Char;
begin
  Result := false;
  S1 := ''; { CPF somente com dgitos }
  for I := 1 to Length(CPF) do begin
    S2 := Copy(CPF, I, 1);
    if Pos(S2, '0123456789') > 0 then
      S1 := S1 + S2;
  end;
  if Length(S1) <> 11 then
    Exit; { No  CPF, pois no so 11 dgitos }

  { Teste se os 11 dg. so iguais }
  B := true;
  C := S1[1];
  for I := 2 to 11 do begin
    B := B and (S1[I] = C);
    C := S1[I];
  end;
  if B then { Todos dg. iguais }
    Exit;

  CalcCPF := Copy(S1, 1, 9);

  { Clculo do 1 dgito }
  Soma := 0;
  for I := 1 to 9 do
    Soma := Soma + StrToInt(Copy(CalcCPF, I, 1)) * (11 - I);
  Digito := 11 - (Soma mod 11);
  if Digito in [ 10, 11 ] then
    CalcCPF := CalcCPF + '0'
  else
    CalcCPF := CalcCPF + IntToStr(Digito);

  { Clculo do 2 dgito }
  Soma := 0;
  for I := 1 to 10 do
    Soma := Soma + StrToInt(Copy(CalcCPF, I, 1)) * (12 - I);
  Digito := 11 - (Soma mod 11);
  if Digito in [ 10, 11 ] then
    CalcCPF := CalcCPF + '0'
  else
    CalcCPF := CalcCPF + IntToStr(Digito);

  if CalcCPF = S1 then
    Result := true;
end;

{ Retorna true se a string for um CGC vlido }
function tbIsCGC(const CGC: string): boolean;
var
  CalcCGC, S1, S2: string;
  I, Soma, Digito: integer;
begin
  Result := false;

  S1 := ''; { CGC somente com dgitos }
  for I := 1 to Length(CGC) do begin
    S2 := Copy(CGC, I, 1);
    if Pos(S2, '0123456789') > 0 then
      S1 := S1 + S2;
  end;
  if Length(S1) <> 14 then
    Exit; { No  CGC, pois no so 14 dgitos }
  if S1 = '00000000000000' then
    Exit;
  CalcCGC := Copy(S1, 1, 12);

  { Clculo do 1 dgito }
  Soma := 0;
  for I := 1 to 4 do
    Soma := Soma + StrToInt(Copy(CalcCGC, I, 1)) * (6 - I);
  for I := 1 to 8 do
    Soma := Soma + StrToInt(Copy(CalcCGC, I + 4, 1)) * (10 - I);
  Digito := 11 - (Soma mod 11);
  if Digito in [ 10, 11 ] then
    CalcCGC := CalcCGC + '0'
  else
    CalcCGC := CalcCGC + IntToStr(Digito);

  { Clculo do 2 dgito }
  Soma := 0;
  for I := 1 to 5 do
    Soma := Soma + StrToInt(Copy(CalcCGC, I, 1)) * (7 - I);
  for I := 1 to 8 do
    Soma := Soma + StrToInt(Copy(CalcCGC, I + 5, 1)) * (10 - I);
  Digito := 11 - (Soma mod 11);
  if Digito in [ 10, 11 ] then
    CalcCGC := CalcCGC + '0'
  else
    CalcCGC := CalcCGC + IntToStr(Digito);

  if CalcCGC = S1 then
    Result := true;
end;

{ Retorna true se a string for CPF ou CGC vlido }
function tbIsCPForCGC(const S: string): boolean;
begin
  Result := tbIsCPF(S) or tbIsCGC(S);
end;

{ Retorna a string formatada como CPF }
function tbFmtCPF(const S: string): string;
var
  Temp: string;
begin
  Result := S;
  Temp := tbDigits(S);
  if Length(Temp) = 11 then
    Result := Copy(Temp,  1, 3) + '.' +
              Copy(Temp,  4, 3) + '.' +
              Copy(Temp,  7, 3) + '-' +
              Copy(Temp, 10, 2);
end;

{ Retorna a string formatada como CGC }
function tbFmtCGC(const S: string): string;
var
  Temp: string;
begin
  Result := S;
  Temp := tbDigits(S);
  if Length(Temp) = 14 then
    Result := Copy(Temp,  1, 2) + '.' +
              Copy(Temp,  3, 3) + '.' +
              Copy(Temp,  6, 3) + '/' +
              Copy(Temp,  9, 4) + '-' +
              Copy(Temp, 13, 2);
end;

{ Retorna a string formatada como CPF se tiver 11 dgitos, ou
  como CGC se tiver 14 dgitos }
function tbFmtCPForCGC(const S: string): string;
var
  Temp: string;
begin
  Result := S;
  Temp := tbDigits(S);
  case Length(Temp) of
    11: Result := tbFmtCPF(S);
    14: Result := tbFmtCGC(S);
  end;
end;

end.
